unit WThread;

interface

uses
  Classes, Messages, Windows,  BZip2, SysUtils, Dialogs, CommCtrl;

type
  WorkThread = class(TThread)
  private
    WNFiles : TStrings;
    WCFiles : TStrings;
    WHosts : TStrings;
    CRPaths : TStrings;
    NRPaths : TStrings;
    WRevision : integer;
    WModHosts : integer;
    WClientSrcDir : string;
    WClientDestDir : string;
    WOutDir : string;
    WClientName : string;
    WClientSknName : string;
    UClientData : string;
    WClientSknDat  : string;
    WClientCustomName: string;
    WClientLinkName: string;
    WClientAppExeName : string;
    WNewsUrl : string;
    UMirrors : TStrings;
    FSource: TStream;
  protected
    procedure Execute; override;
    procedure PackFiles(UploadDir: string; LRPaths,FList: Tstrings);
    procedure EnableForm;
    procedure CompressFile(const ASource, ADest: TFileName);
    procedure DoCompressStream(ASource, ADest: TStream);
    procedure BZProgress(Sender: TObject);
    procedure CreateList;
    procedure CreateSettingsLst;
    procedure UpdateProgress;
    procedure UpdateFileProgressMax;
    procedure UpdateFileProgress;
  public
    property RevisionNum : integer write WRevision;
    property UpdaterClienName : string write WClientName;
    property UpdaterSkinName : string write WClientSknName;
    property OutputDir : string write WOutDir;
    property ModHosts : integer write WModHosts;
    property NormalFiles : TStrings write WNFiles;
    property CriticalFiles : TStrings write WCFiles;
    property HostsLines : TStrings write WHosts;
    property ClientSrcDir : string write WClientSrcDir;
    property ClientCustomName : string write WClientCustomName;
    property ClientLinkName : string write WClientLinkName;
    property ClientAppExeName : string write WClientAppExeName;
    property ClientDestDir : string write WClientDestDir;
    property ClientNewsUrl : string write WNewsUrl;
    property UpdatesMirrorList : TStrings write UMirrors;
    property RelativePathsNormal : TStrings write NRPaths;
    property RelativePathsCritical : TStrings write CRPaths;
  end;

implementation
uses Unit1, Unit2, md5, Misc, datfile;
//var str:string;

procedure WorkThread.EnableForm;
begin
  Form1.Enabled:=true;
  ShowMessage(' Complete ! ');
end;

procedure WorkThread.UpdateProgress;
begin
  Form1.ProgressBar2.Position := Form1.ProgressBar2.Position + 1;
end;

procedure WorkThread.UpdateFileProgress;
begin
  Form1.ProgressBar1.Position := FSource.Position;
end;

procedure WorkThread.UpdateFileProgressMax;
begin
  Form1.ProgressBar1.Max:= FSource.Size;
end;

procedure WorkThread.BZProgress(Sender: TObject);
begin
  Synchronize(UpdateFileProgress);
end;

//  0 ^^



procedure WorkThread.PackFiles(UploadDir: string; LRPaths, FList: Tstrings);
var
  FileN,hash: string;
  ZipSize,i :integer;
begin
  if LRPaths.Count>0 then
  begin
    for i:=0 to FList.Count-1 do
    begin
      FileN:= UploadDir+'\'+LRPaths[i]+'.bz2';
      hash:= md5.MD5Print(md5.MD5File(FList[i]));
      if(not DirectoryExists(ExtractFilePath(FileN))) then
            ForceDirectories(ExtractFilePath(FileN));
      CompressFile(FList[i],FileN);
      ZipSize:=GetFileSize(FileN);
      FList[i]:=LRPaths[i]+'|'+hash+'|'+IntToStr(ZipSize);
      Synchronize(UpdateProgress);
    end;
   end;
end;

procedure WorkThread.CreateList;
var
  LstFile : Textfile;
  i : integer;
begin
AssignFile(LstFile,WOutDir+'\files.lst');
Rewrite(LstFile);
WriteLn(LstFile,'[settings]');
WriteLn(LstFile,'Rev|'+IntToStr(WRevision));
WriteLn(LstFile,'ModHosts|'+IntToStr(WModHosts));
WriteLn(LstFile,'[/settings]');
WriteLn(LstFile,'[self]');
WriteLn(LstFile,UClientData);
WriteLn(LstFile,WClientSknDat);
WriteLn(LstFile,'[/self]');
  if WModHosts>0 then
  begin
  WriteLn(LstFile,'[hosts]');
    for i:=0 to WHosts.Count-1 do
    begin
      WriteLn(LstFile,ReplaceStr(WHosts[i],'=','|'));
    end;
  WriteLn(LstFile,'[/hosts]');
  end;
WriteLn(LstFile,'[files_critical]');
  for i:=0 to WCFiles.Count-1 do
  begin
    WriteLn(LstFile,WCFiles[i]);
  end;
WriteLn(LstFile,'[/files_critical]');
WriteLn(LstFile,'[files_normal]');
  for i:=0 to WNFiles.Count-1 do
    WriteLn(LstFile,WNFiles[i]);
WriteLn(LstFile,'[/files_normal]');
CloseFile(LstFile);
CompressFile(WOutDir+'\files.lst',WOutDir+'\files.lst.bz2');
DeleteFile(WOutDir+'\files.lst');
end;

procedure WorkThread.CreateSettingsLst;
var
  LstFile : Textfile;
  i : integer;
begin
AssignFile(LstFile,WClientDestDir+'\settings.lst');
Rewrite(LstFile);
WriteLn(LstFile,'[settings]');
WriteLn(LstFile,'NewsUrl|'+WNewsUrl);
if (WClientLinkName='') then WClientLinkName:='none';
WriteLn(LstFile,'LinkName|'+WClientLinkName);
WriteLn(LstFile,'RunCustom|'+WClientAppExeName);
WriteLn(LstFile,'[/settings]');
WriteLn(LstFile,'[updates_mirrors]');
 for i:=0 to UMirrors.Count-1 do
  WriteLn(LstFile,ReplaceStr(Umirrors[i],'=','|'));
WriteLn(LstFile,'[/updates_mirrors]');
CloseFile(LstFile);
end;

procedure WorkThread.DoCompressStream(ASource, ADest: TStream);
var
  Comp: TBZCompressionStream;
begin
  FSource := ASource;
  Synchronize(UpdateFileProgressMax);
  Comp := TBZCompressionStream.Create(bs9, ADest);
  try
    Comp.OnProgress := BZProgress;
    Comp.CopyFrom(ASource, 0);
  finally
    Comp.Free;
    FSource := nil;
  end;
end;


procedure WorkThread.CompressFile(const ASource, ADest: TFileName);
var
  Source, Dest: TStream;
begin
  Source := TFileStream.Create(ASource, fmOpenRead + fmShareDenyWrite);
  try
    Dest := TFileStream.Create(ADest, fmCreate);
    try
      DoCompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

procedure WorkThread.Execute;
var
  WNptr : ^TStringList;
  WCptr : ^TStringList;
  ClientPath, SknPath, Skn:string;
begin
 WNptr:= @WNFiles;
 WCptr:= @WCFiles;
 ClientPath:= WClientSrcDir;
 if (WClientDestDir<>'') then
 begin
  CopyFile(PChar(ClientPath),PChar(WClientDestDir+'\'+WClientCustomName), false);
  Skn:=ChangeFileExt(WClientDestDir+'\'+WClientCustomName,'.l3s');
  CopyFile(PChar(WClientSknName),PChar(Skn), false);
  CreateSettingsLst;
  CompressFile(WClientDestDir+'\settings.lst',WClientDestDir+'\settings');
  DeleteFile(WClientDestDir+'\settings.lst');
  SaveToEnd(Skn,WClientDestDir+'\settings',true);
 end;
 if (FileExists(ClientPath)) and (WOutDir <> '') then
 begin
    CompressFile(ClientPath,WOutDir+'\'+WClientCustomName+'.bz2');
    CompressFile(Skn,WOutDir+'\'+ExtractFileName(Skn)+'.bz2');
    UClientData:=WClientCustomName+'|'+ md5.MD5Print(md5.MD5File(ClientPath))+'|'+IntToStr(GetFileSize(WOutDir+'\'+WClientCustomName+'.bz2'));
    WClientSknDat:= ExtractFileName(Skn)+'|'+ md5.MD5Print(md5.MD5File(Skn))+'|'+IntToStr(GetFileSize(WOutDir+'\'+ExtractFileName(Skn)+'.bz2'));
 end;
 if (WOutDir <> '') then
 begin
   PackFiles(WOutDir, NRPaths , WNptr^);
   PackFiles(WOutDir, CRPaths , WCptr^);
   CreateList;
 end;
 Synchronize(EnableForm);
end;

end.
